/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WKUTTA_H
#define WKUTTA_H

#include "dart.h"
#include "wGroups.h"
#include "wElement.h"
#include "wNode.h"
#include "wKuttaElement.h"
#include <vector>
#include <string>

namespace dart
{

/**
 * @brief Group of elements to apply the Kutta condition
 * @authors Adrien Crovato
 */
class DART_API Kutta : public tbox::Groups
{
public:
    std::vector<std::pair<tbox::Node *, tbox::Node *>> nodMap; ///< upper to lower trailing edge nodes map
#ifndef SWIG
    std::vector<KuttaElement *> kEle; ///< list of Kutta elements
#endif

    Kutta(std::shared_ptr<tbox::MshData> _msh, std::vector<int> const &nos);
    Kutta(std::shared_ptr<tbox::MshData> _msh, std::vector<std::string> const &names);
    virtual ~Kutta();

#ifndef SWIG
    virtual void write(std::ostream &out) const override;
#endif

private:
    void connectNodes();
    void createElements();
    void connectElements(std::vector<std::pair<tbox::Element *, tbox::Element *>> &svPair, std::vector<std::vector<std::pair<size_t, tbox::Node *>>> &surN, bool uFlag);
};

} // namespace dart

#endif // WKUTTA_H
